@@ -117,4 +117,9 @@ gboolean volatile cancel_jukebox_operation;
 /* Global debug level variable (standard = 7) */
 gint gnomad_debug;
 
+#if GTK_CHECK_VERSION(2,6,0)
+/* A proc for hiding dialog windows */
+GCallback dispose_of_dialog_window(GtkButton * button, gpointer data);
+#endif
+
 #endif
