@@ -21,7 +21,10 @@ Boston, MA 02111-1307, USA.
 #include "jukebox.h"
 #include "data.h"
 #include "util.h"
+
+#if !GTK_CHECK_VERSION(2,6,0)
 #include <libgnomeui/libgnomeui.h>
+#endif
 
 static GtkWidget *new_folder_dialog;
 static GtkWidget *new_folder_entry;
@@ -42,6 +45,7 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
   if (metalist != NULL) {
     GtkWidget *label1, *label2, *dialog, *button, *separator;
 
+#if !GTK_CHECK_VERSION(2,6,0)
     dialog = gnome_dialog_new(_("Transfering files from jukebox file storage"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
@@ -49,6 +53,19 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
 				0,
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
+#else
+    dialog = gtk_dialog_new_with_buttons(_("Transfering files from jukebox file storage"),
+					 NULL,
+					 0,
+					 NULL);
+    button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+				   GTK_STOCK_CANCEL,
+				   0);
+    g_signal_connect(button,
+		     "clicked",
+		     G_CALLBACK(cancel_jukebox_operation_click),
+		     NULL);
+#endif
     g_signal_connect_object(GTK_OBJECT(dialog),
 			    "delete_event",
 			    G_CALLBACK(gtk_widget_destroy),
@@ -59,10 +76,17 @@ static GCallback data_from_jukebox_to_hd (GtkButton *button,
     label2 = gtk_label_new("");
     progress_bar = gtk_progress_bar_new();
     separator = gtk_hseparator_new();
+#if !GTK_CHECK_VERSION(2,6,0)
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label2, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), separator, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), progress_bar, TRUE, TRUE, 0);
+#else
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label2, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), separator, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), progress_bar, TRUE, TRUE, 0);
+#endif
     gtk_widget_show_all(dialog);
     
     jb2hd_data_thread_args.dialog = dialog;
@@ -89,7 +113,7 @@ static GCallback data_from_hd_to_jukebox()
   
   if (metalist != NULL) {
     GtkWidget *label1, *label2, *dialog, *button, *separator;
-
+#if !GTK_CHECK_VERSION(2,6,0)
     dialog = gnome_dialog_new(_("Transfering files to jukebox file storage"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
@@ -97,6 +121,20 @@ static GCallback data_from_hd_to_jukebox()
 				0,
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
+#else
+    dialog=gtk_dialog_new_with_buttons(_("Transfering files to jukebox file storage"),
+		    NULL,
+		    0,
+		    NULL);
+    
+    button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+		    GTK_STOCK_CANCEL,
+		    0);
+    g_signal_connect(button,
+		    "clicked",
+		    G_CALLBACK(cancel_jukebox_operation_click),
+		    NULL);
+#endif
     g_signal_connect_object(GTK_OBJECT(dialog),
 			    "delete_event",
 			    G_CALLBACK(gtk_widget_destroy),
@@ -107,10 +145,17 @@ static GCallback data_from_hd_to_jukebox()
     label2 = gtk_label_new("");
     progress_bar = gtk_progress_bar_new();
     separator = gtk_hseparator_new();
+#if !GTK_CHECK_VERSION(2,6,0)
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), label2, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), separator, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), progress_bar, TRUE, TRUE, 0);
+#else
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label1, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), label2, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), separator, TRUE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), progress_bar, TRUE, TRUE, 0);
+#endif
     gtk_widget_show_all(dialog);
     
     hd2jb_data_thread_args.dialog = dialog;
@@ -297,6 +342,7 @@ static GCallback jbmenu_create_folder_response(gpointer data)
 {
   GtkWidget *label, *button;
 
+#if !GTK_CHECK_VERSION(2,6,0)
   new_folder_dialog = gnome_dialog_new(_("Create a new folder"),
 				       GNOME_STOCK_BUTTON_CANCEL,
 				       GNOME_STOCK_BUTTON_OK,
@@ -309,13 +355,43 @@ static GCallback jbmenu_create_folder_response(gpointer data)
 			      1,
 			      GTK_SIGNAL_FUNC(create_new_folder_ok_button),
 			      NULL);
+#else
+  new_folder_dialog=gtk_dialog_new_with_buttons(_("Create a new folder"),
+		  NULL,
+		  0,
+		  NULL);
+
+  button = gtk_dialog_add_button(GTK_DIALOG(new_folder_dialog),
+		  GTK_STOCK_CANCEL,
+		  0);
+  g_signal_connect(button,
+		  "clicked",
+		  GTK_SIGNAL_FUNC(dispose_of_dialog_window),
+		  GTK_OBJECT(new_folder_dialog));
+
+  button = gtk_dialog_add_button(GTK_DIALOG(new_folder_dialog),
+		  GTK_STOCK_OK,
+		  1);
+  g_signal_connect(button,
+		  "clicked",
+		  GTK_SIGNAL_FUNC(create_new_folder_ok_button),
+		  NULL);	  
+#endif
   gtk_window_set_position (GTK_WINDOW(new_folder_dialog), GTK_WIN_POS_MOUSE);
 
   label = gtk_label_new(_("Choose a name for the new folder:"));
+#if !GTK_CHECK_VERSION(2,6,0)
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(new_folder_dialog)->vbox), label, TRUE, TRUE, 0);
+#else
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_folder_dialog)->vbox), label, TRUE, TRUE, 0);
+#endif
   gtk_widget_show(label);
   new_folder_entry = gtk_entry_new();
+#if !GTK_CHECK_VERSION(2,6,0)
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(new_folder_dialog)->vbox), new_folder_entry, TRUE, TRUE, 0);
+#else
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_folder_dialog)->vbox), new_folder_entry, TRUE, TRUE, 0);
+#endif
   gtk_widget_show(new_folder_entry);
 
   g_signal_connect(G_OBJECT (new_folder_dialog), 
