@@ -24,6 +24,6 @@ Boston, MA 02111-1307, USA.
 
 /* Exported functions */
 gchar *compose_filename(metadata_t *meta);
-void info_from_path(metadata_t *meta);
+void info_from_path(metadata_t *meta, gboolean be_clever);
 
 #endif
