@@ -34,11 +34,13 @@ Boston, MA 02111-1307, USA.
 /* Extra includes used by this file only */
 
 #include <unistd.h>
-#include <pwd.h>
 #include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#ifndef G_OS_WIN32
+#include <pwd.h>
+#endif
 
 /* Globally imported variables */
 extern GtkWidget *main_window;
@@ -51,44 +53,53 @@ static gint preffd;
 gboolean copy_file(gchar *source, gchar *dest)
 {
   gint f1, f2;
-  gboolean retval = FALSE;
-  
-  f1 = (gint) open(source, O_RDONLY, 0);
+
+#if !GLIB_CHECK_VERSION(2,6,0)  
+  f1 = (gint) open(source, READONLY_FLAGS, 0);
+#else
+  f1 = (gint) g_open(source, READONLY_FLAGS, 0);
+#endif
   if (f1 < 0)
     return FALSE;
 
+#if !GLIB_CHECK_VERSION(2,8,0)  
   f2 = (gint) creat(dest, (mode_t) CREATE_FILEMODE);
+#else
+  f2 = (gint) g_creat(dest, (mode_t) CREATE_FILEMODE);
+#endif
   if (f2 >= 0) {
     guint bufsiz;
+    gboolean retval = FALSE;
+    gchar *buffer;
     
     for (bufsiz = 0x8000; bufsiz >= 128; bufsiz >>= 1) {
-      register gchar *buffer;
-      
       buffer = (gchar *) g_malloc(bufsiz);
-      if (buffer) {
-	while (1) {
-	  register guint n;
-	  
-	  n = read(f1,buffer,bufsiz);
-	  if (n == -1)
-	    break;
-	  if (n == 0) {
-	    retval = TRUE;
-	    break;
-	  }
-	  if (n != write(f2,buffer,n))
-	    break;
-	}
-	g_free(buffer);
+    }
+
+    if (buffer == NULL) {
+      return FALSE;
+    }
+
+    while (1) {
+      register gint n;
+      
+      n = read(f1,buffer,bufsiz);
+      if (n == -1) {
+	break;
+      }
+      if (n == 0) {
+	retval = TRUE;
+	break;
+      }
+      if (n != write(f2,buffer,n)) {
 	break;
       }
     }
-    if (retval) {
-      close(f1);
-      close(f2);
-    } else {
-      close(f1);
-      close(f2);
+
+    g_free(buffer);
+    close(f1);
+    close(f2);
+    if (!retval) {
       remove(dest); // Delete partial file
     }
     return retval;
@@ -97,11 +108,30 @@ gboolean copy_file(gchar *source, gchar *dest)
   return FALSE;
 }
 
+/**
+ * Get the 64bit file size for a file. Return 0 if failed.
+ * Does not currently expect path to be UTF-8 but assumes
+ * local filesystem encoding.
+ */
+guint64 get_64bit_file_size(const gchar * const path)
+{
+  struct stat sb;
+  
+  // Get the size that we actually have now...
+  if ( stat(path, &sb) == -1 ) {
+    g_print("Error getting file size for \"%s\"!!\n", path);
+    return 0;
+  }
+  // TODO: fix this to actually find the 64bit size!
+  return (guint64) sb.st_size;
+}
+
 /* Unicode conversion wrappers that enables
  * forcing the local charset to something disregarding
  * the locale setting and such stuff. */
-gchar *filename_toutf8(const gchar *instr)
+gchar *filename_toutf8(const gchar * const instr)
 {
+#ifndef G_OS_WIN32
   gchar *tmp;
 
   if (get_prefs_force_8859()) {
@@ -120,10 +150,15 @@ gchar *filename_toutf8(const gchar *instr)
     }
   }
   return tmp;
+#else
+  // Windows will always use UTF-8 everywhere.
+  return g_strdup(instr);
+#endif
 }
 
-gchar *filename_fromutf8(const gchar *instr)
+gchar *filename_fromutf8(const gchar * const instr)
 {
+#ifndef G_OS_WIN32
   gchar *tmp;
   
   if (get_prefs_force_8859()) {
@@ -132,6 +167,10 @@ gchar *filename_fromutf8(const gchar *instr)
     tmp = g_filename_from_utf8(instr,-1,NULL,NULL,NULL);
   }
   return tmp;
+#else
+  // Windows will always use UTF-8 everywhere.
+  return g_strdup(instr);
+#endif
 }
 
 gboolean is_directory(gchar *dir) {
@@ -173,7 +212,7 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
       int is_file;
 
       strncpy(path_buffer, tmppath, 512);
-      strcat(path_buffer, "/");
+      strcat(path_buffer, G_DIR_SEPARATOR_S);
       strcat(path_buffer, dir_entry->d_name);
       if (stat(path_buffer, &entry_status)) {
 	if (gnomad_debug != 0) {
@@ -191,26 +230,32 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
 	}
 	/* If the prefs are selected not to display the hidden
 	 * directories, we hide them. */
+#ifndef G_OS_WIN32
 	if (strlen(dir_entry->d_name) >= 2) {
 	  if (dir_entry->d_name[0] == '.' &&
 	      dir_entry->d_name[1] != '.' &&
 	      !get_prefs_display_dotdirs())
 	    continue;
 	}
+#endif
       } else if (is_file) {
 	char tmp_codec[4];
+	char *tmp;
 	int i;
 
 	if (!anyfile) {
 	  if (strlen(dir_entry->d_name) < 4)
 	    continue;
 	  strcpy(tmp_codec, dir_entry->d_name+strlen(dir_entry->d_name)-3);
-	  g_strdown(tmp_codec);
+	  tmp = g_utf8_strdown(tmp_codec, -1);
 	  /* Only display valid files */
-	  if (g_ascii_strcasecmp(tmp_codec,"mp3") &&
-	      g_ascii_strcasecmp(tmp_codec,"wav") &&
-	      g_ascii_strcasecmp(tmp_codec,"wma"))
+	  if (g_ascii_strcasecmp(tmp,"mp3") &&
+	      g_ascii_strcasecmp(tmp,"wav") &&
+	      g_ascii_strcasecmp(tmp,"wma")) {
+	    g_free(tmp);
 	    continue;
+	  }
+	  g_free(tmp);
 	}
       }
       /* only directories and valid files come here, the
@@ -220,7 +265,13 @@ static GList *get_directory_listing(gchar* base_path, gboolean anyfile)
 	/* For directories, set size to 0 */
 	size = 0;
       }
+#ifndef G_OS_WIN32
       filename = g_strdup(path_buffer);
+#else
+      // On Windows we need to convert into UTF-8 since
+      // this does not use the stdio wrappers properly.
+      filename = g_locale_to_utf8(path_buffer,-1,NULL,NULL,NULL);
+#endif
       if (gnomad_debug != 0) {
 	g_print("Adding dir entry: %s, size %d\n", filename, size);
       }
@@ -490,6 +541,11 @@ void fill_in_dir(listtype_t listtype, gchar *path)
   dirfill_thread_arg_t *dirfill_thread_args;
   GtkWidget *label1;
   GtkWidget *dialog, *scanlabel, *metadata_progress_bar, *separator;
+#ifdef G_OS_WIN32
+  GList *metalist = NULL;
+  GList *tmp;
+#endif
+
   dirfill_thread_args = (dirfill_thread_arg_t *) g_malloc(sizeof(dirfill_thread_arg_t));
 
   // Create a dialog...
@@ -553,9 +609,32 @@ void fill_in_dir(listtype_t listtype, gchar *path)
   dirfill_thread_args->listtype = listtype;
   dirfill_thread_args->path = g_strdup(path);
   dirfill_thread_args->cancelled = FALSE;
-    
+
+#ifndef G_OS_WIN32
   // TODO: check no reentrance while this thread is running...
   g_thread_create(dirfill_thread,(gpointer) dirfill_thread_args, FALSE, NULL);
+#else
+  // Windows cannot do all things in the threads. Therefore we just unthread
+  // it. This is a bit time consuming...
+  metalist = get_metadata_dir(dirfill_thread_args->listtype, dirfill_thread_args->path, NULL);
+
+  gtk_label_set_text(GTK_LABEL(dirfill_thread_args->label), _("Adding metadata to view"));
+  recreate_list_store(dirfill_thread_args->listtype);
+  tmp = metalist;
+  while (tmp) {
+    metadata_t *meta;
+    
+    meta = (metadata_t *) tmp->data;
+    add_metadata_to_model(meta, dirfill_thread_args->listtype);
+    tmp = tmp->next;
+    destroy_metadata_t(meta);
+  }
+  view_and_sort_list_store(dirfill_thread_args->listtype);
+  gtk_widget_destroy(dirfill_thread_args->dialog);
+  g_list_free(metalist);
+  g_free(dirfill_thread_args->path);
+  g_free(dirfill_thread_args);
+#endif
 }
 
 gchar *get_current_dir()
@@ -574,7 +653,11 @@ void change_directory(gchar *dir)
   gchar *tmp;
 
   tmp = filename_fromutf8(dir);
+#if GLIB_CHECK_VERSION(2,8,0)
+  g_chdir(tmp);
+#else
   chdir(tmp);
+#endif
   g_free(tmp);
 }
 
@@ -587,14 +670,18 @@ void delete_files(GList *filelist)
     metadata_t *meta = (metadata_t *) tmplist->data;
     /* g_print("Deleting: %s\n", meta->path); */
     tmp = filename_fromutf8(meta->path);
+#if !GLIB_CHECK_VERSION(2,6,0)
     unlink(tmp);
+#else
+    g_unlink(tmp);
+#endif
     g_free(tmp);
     tmplist = tmplist->next;
   }
 }
 
 /* Copy an MP3 file and remove the tag */
-gboolean clone_and_strip_id3(gchar *source, gchar *dest)
+gboolean clone_and_strip_id3(const gchar * const source, const gchar * const dest)
 {
   gchar *tmpsource, *tmpdest;
 
@@ -620,18 +707,33 @@ gint get_prefsfile_fd(gboolean write)
   static gchar *path = NULL;
 
   if (path == NULL) {
-    g_strdown(filename);
+    gchar *tmp;
+
+    tmp = g_utf8_strdown(filename, -1);
     path = g_strdup(g_get_home_dir());
     path = stringcat(path, G_DIR_SEPARATOR_S);
-    path = stringcat(path, filename);
+    path = stringcat(path, tmp);
+    g_free(tmp);
   }
   if (write) {
-    preffd = (gint) open(path, O_WRONLY, (mode_t) CREATE_FILEMODE);
+#if !GLIB_CHECK_VERSION(2,6,0)
+    preffd = (gint) open(path, WRITEONLY_FLAGS, (mode_t) CREATE_FILEMODE);
+#else
+    preffd = (gint) g_open(path, WRITEONLY_FLAGS, (mode_t) CREATE_FILEMODE);
+#endif
     if (preffd < 0)
+#if !GLIB_CHECK_VERSION(2,8,0)
       preffd = (gint) creat(path, (mode_t) CREATE_FILEMODE);
+#else
+      preffd = (gint) g_creat(path, (mode_t) CREATE_FILEMODE);
+#endif
   }
   else
-    preffd = (gint) open(path, O_RDONLY, 0);
+#if !GLIB_CHECK_VERSION(2,6,0)
+    preffd = (gint) open(path, READONLY_FLAGS, 0);
+#else
+    preffd = (gint) g_open(path, READONLY_FLAGS, 0);
+#endif
   if (preffd < 0 &&
       write)
     create_error_dialog(_("Could not write preference file"));
@@ -656,6 +758,8 @@ void rewrite_prefsfile(gchar *prefs)
 /* Function from gftp lib/misc.c */
 char *expand_path (char *src)
 {
+// The following is very POSIX
+#ifndef G_OS_WIN32
   char *str, *pos, *endpos, *prevpos, *newstr, *tempstr, tempchar;
   struct passwd *pw;
 
@@ -663,33 +767,31 @@ char *expand_path (char *src)
 
   str = filename_fromutf8(src);
 
-  if (*str == '~')
-    {
-      if (*(str + 1) == '/' || (pos = strchr (str, '/')) == NULL)
-	pw = getpwuid (geteuid ());
-      else
-	{
-	  *pos = '\0';
-	  pw = getpwnam (str + 1);
-	  *pos = '/';
-	}
+  if (*str == '~') {
+    if (*(str + 1) == G_DIR_SEPARATOR || (pos = strchr (str, G_DIR_SEPARATOR)) == NULL) {
+      pw = getpwuid (geteuid ());
+    } else {
+      *pos = '\0';
+      pw = getpwnam (str + 1);
+      *pos = G_DIR_SEPARATOR;
     }
+  }
 
   endpos = str;
   newstr = NULL;
-  while ((pos = strchr (endpos, '/')) != NULL)
+  while ((pos = strchr (endpos, G_DIR_SEPARATOR)) != NULL)
     {
       pos++;
-      while (*pos == '/')
+      while (*pos == G_DIR_SEPARATOR)
 	pos++;
-      if ((endpos = strchr (pos, '/')) == NULL)
+      if ((endpos = strchr (pos, G_DIR_SEPARATOR)) == NULL)
 	endpos = strchr (pos, '\0');
       tempchar = *endpos;
       *endpos = '\0';
       if (strcmp (pos, "..") == 0)
 	{
 	  *(pos - 1) = '\0';
-	  if (newstr != NULL && (prevpos = strrchr (newstr, '/')) != NULL)
+	  if (newstr != NULL && (prevpos = strrchr (newstr, G_DIR_SEPARATOR)) != NULL)
 	    *prevpos = '\0';
 	}
       else if (strcmp (pos, ".") != 0)
@@ -717,7 +819,7 @@ char *expand_path (char *src)
       if (newstr != NULL)
 	g_free (newstr);
       newstr = g_malloc (2);
-      *newstr = '/';
+      *newstr = G_DIR_SEPARATOR;
       *(newstr + 1) = '\0';
     }
 
@@ -729,7 +831,7 @@ char *expand_path (char *src)
 
   if (pw != NULL)
     {
-      if ((pos = strchr (str, '/')) == NULL)
+      if ((pos = strchr (str, G_DIR_SEPARATOR)) == NULL)
 	{
 	  newstr = g_malloc (strlen (pw->pw_dir) + 1);
 	  strcpy (newstr, pw->pw_dir);
@@ -750,6 +852,9 @@ char *expand_path (char *src)
   newstr = filename_toutf8(str);
   g_free(str);
   return (newstr);
+#else
+  return g_strdup(src);
+#endif
 }
 
 /* Test if a file is writeable or not */
@@ -758,7 +863,12 @@ gboolean test_writeable(gchar *path)
   FILE *file;
 
   /* Test to know if we can write into the file */
-  if ((file = fopen(path,"r+"))==NULL ) {
+#if !GLIB_CHECK_VERSION(2,6,0)
+  file = fopen(path,READPLUS_FOPEN_STRING);
+#else
+  file = (FILE *) g_fopen(path,READPLUS_FOPEN_STRING);
+#endif
+  if (file == NULL) {
     return FALSE;
   }
   fclose(file);
@@ -767,7 +877,11 @@ gboolean test_writeable(gchar *path)
 
 gboolean create_directory(gchar *path)
 {
+#if !GLIB_CHECK_VERSION(2,6,0)
   if (mkdir(path, CREATE_DIRMODE) == -1) {
+#else
+  if (g_mkdir(path, CREATE_DIRMODE) == -1) {
+#endif
     return FALSE;
   } else {
     return TRUE;
@@ -813,9 +927,17 @@ void export_playlistfile(gchar *filename, gchar *formatstring, GList *playlist,
   GList *tmp;
   guint i;
 
-  fd = (gint) open(tmpname, O_WRONLY, (mode_t) CREATE_FILEMODE);
+#if !GLIB_CHECK_VERSION(2,6,0)
+  fd = (gint) open(tmpname, WRITEONLY_FLAGS, (mode_t) CREATE_FILEMODE);
+#else
+  fd = (gint) g_open(tmpname, WRITEONLY_FLAGS, (mode_t) CREATE_FILEMODE);
+#endif
   if (fd < 0)
+#if !GLIB_CHECK_VERSION(2,8,0)
     fd = (gint) creat(tmpname, (mode_t) CREATE_FILEMODE);
+#else
+    fd = (gint) g_creat(tmpname, (mode_t) CREATE_FILEMODE);
+#endif
   if (fd < 0) {
     create_error_dialog(_("Could not write exported playlist"));
     return;
