@@ -458,7 +458,7 @@ int main(int argc,
   extern char *optarg;
   gint opt;
   gint i;
-  GError *icon_load_error;
+  GError *icon_load_error = NULL;
 
   /* Binds for internationalization (i18n) */
   bindtextdomain (GETTEXT_PACKAGE, GNOMADLOCALEDIR);
