@@ -20,9 +20,19 @@ Boston, MA 02111-1307, USA.
 #ifndef JUKEBOXH_INCLUDED
 #define JUKEBOXH_INCLUDED 1
 
-/* The jukebox library is included into all files
- * using this file */
+/*
+ * The jukebox library is included into all files
+ * using this file
+ */
 #include <libnjb.h>
+
+/*
+ * libmtp is included conditionally.
+ */
+#ifdef HAVE_LIBMTP
+#include <libmtp.h>
+#endif
+
 #include "metadata.h"
 
 typedef struct 
@@ -107,10 +117,8 @@ guint jukebox_randomize_playlist(guint playlist, GtkTreeStore *pltreestore);
 guint jukebox_delete_track_from_playlist(guint trackid, guint plist, GtkTreeStore *pltreestore);
 gchar *jukebox_get_ownerstring(void);
 void jukebox_set_ownerstring(gchar *owner);
-#ifdef G_HAVE_GINT64
 void jukebox_getusage(guint64 *total, guint64 *free, guint64 *used, 
 		      guint *songs, guint *playlists, guint *datafiles);
-#endif
 gchar *jukebox_get_time(void);
 void jukebox_reset_get_eax(void);
 njb_eax_t *jukebox_get_eax(void);
