@@ -61,7 +61,7 @@ void destroy_metadata_t(metadata_t *meta)
   g_free(meta);
 }
 
-metadata_t *clone_metadata_t(metadata_t *meta)
+metadata_t *clone_metadata_t(metadata_t const * const meta)
 {
   metadata_t *clone = new_metadata_t();
 
@@ -89,36 +89,57 @@ metadata_t *clone_metadata_t(metadata_t *meta)
   return clone;
 }
 
-void dump_metadata_t(metadata_t *meta)
+static gchar *metadata_string_escape(gchar *str)
 {
-  g_print("\n---- Song metadata -----\n");
-  g_print("Artist:    %s\n", meta->artist);
-  g_print("Title:     %s\n", meta->title);
-  g_print("Album:     %s\n", meta->album);
-  g_print("Year:      %lu\n", meta->year);
-  g_print("Genre:     %s\n", meta->genre);
-  g_print("Length:    %s\n", meta->length);
-  g_print("Size:      %lu bytes\n", meta->size);
-  g_print("Codec:     %s\n", meta->codec);
-  g_print("Track#:    %lu\n", meta->trackno);
-  g_print("Protected: %s\n", meta->protect ? "YES" : "NO");
-  g_print("Filename:  %s\n", meta->filename);
-  g_print("Path:      %s\n", meta->path);
-  g_print("Folder:    %s\n", meta->folder);
+  static gchar null_value[] = "(null)";
+
+  if (str == NULL) {
+    return null_value;
+  }
+  return str;
 }
 
-static void strip_filename_illegal_chars(gchar *str)
+void dump_metadata_t(metadata_t const * const meta)
+{
+  g_print("\n---- Song metadata -----\n");
+  g_print("Artist:    %s\n", metadata_string_escape(meta->artist));
+  g_print("Title:     %s\n", metadata_string_escape(meta->title));
+  g_print("Album:     %s\n", metadata_string_escape(meta->album));
+  g_print("Year:      %lu\n", meta->year);
+  g_print("Genre:     %s\n", metadata_string_escape(meta->genre));
+  g_print("Length:    %s\n", metadata_string_escape(meta->length));
+  g_print("Size:      %lu bytes\n", meta->size);
+  g_print("Codec:     %s\n", metadata_string_escape(meta->codec));
+  g_print("Track#:    %lu\n", meta->trackno);
+  g_print("Protected: %s\n", meta->protect ? "YES" : "NO");
+  g_print("Filename:  %s\n", metadata_string_escape(meta->filename));
+  g_print("Path:      %s\n", metadata_string_escape(meta->path));
+  g_print("Folder:    %s\n", metadata_string_escape(meta->folder));
+}
+
+static void strip_filename_illegal_chars(gchar * const str)
 {
   guint i;
 
   for (i = 0; i < strlen(str); i++) {
-    if (str[i] == G_DIR_SEPARATOR) {
+    if (str[i] == G_DIR_SEPARATOR || str[i] == G_SEARCHPATH_SEPARATOR) {
       str[i] = '_';
     }
   }
 }
 
-gchar *compose_metadata_string(metadata_t *meta, gchar *template, gboolean filesystemsafe)
+/**
+ * This function creates a metadata string to be used for
+ * e.g. filenames using a template.
+ * @param meta the metadata to use for composing the string
+ * @param template a template string giving the metadata arrangement
+ * @param filesystemsafe whether some illegal characters that cannot
+ *        be used in the underlying filesystem should be stripped from
+ *        the string.
+ */
+gchar *compose_metadata_string(metadata_t const * const meta, 
+			       gchar const * const template, 
+			       gboolean const filesystemsafe)
 {
   gchar *tmp;
   gchar *tmpyear;
@@ -171,6 +192,7 @@ gchar *compose_metadata_string(metadata_t *meta, gchar *template, gboolean files
   } else {
     returnname = replacestring(returnname, "%g", "<Unknown>");
   }
+  // The following does not contain filesystem-illegal characters
   tmptrackno = g_strdup_printf("%02lu", meta->trackno);
   if (tmptrackno) {
     returnname = replacestring(returnname, "%n", tmptrackno);
@@ -228,7 +250,8 @@ metadata_t *get_data_metadata_from_model(GtkTreeModel *model,
 }
 
 
-void add_metadata_to_model(metadata_t *meta, listtype_t listtype)
+void add_metadata_to_model(metadata_t const * const meta, 
+			   listtype_t const listtype)
 {
   GtkListStore *liststore;
   GtkTreeIter listiter;
@@ -445,10 +468,67 @@ void remove_selected(listtype_t listtype)
   }
 }
 
+gint sort_iter_compare_func(GtkTreeModel *model,
+     GtkTreeIter  *a,
+     GtkTreeIter  *b,
+     gpointer      userdata)
+{
+    gint sortcol = GPOINTER_TO_INT(userdata);
+    gint ret = 0;
+    gchar *name1 = NULL, *name2 = NULL;
+    gchar *subname1 = NULL, *subname2 = NULL;
+    guint  subint1 = 0, subint2 = 0;
+    gtk_tree_model_get(model, a, sortcol, &name1, -1);
+    gtk_tree_model_get(model, b, sortcol, &name2, -1);
+
+    switch (sortcol)
+    {
+      case TRACKLIST_ARTIST_COLUMN:
+      {
+        gtk_tree_model_get(model, a, TRACKLIST_TITLE_COLUMN, &subname1, -1);
+        gtk_tree_model_get(model, b, TRACKLIST_TITLE_COLUMN, &subname2, -1);
+	break;
+      }
+      case TRACKLIST_TITLE_COLUMN:
+      {
+        gtk_tree_model_get(model, a, TRACKLIST_ARTIST_COLUMN, &subname1, -1);
+        gtk_tree_model_get(model, b, TRACKLIST_ARTIST_COLUMN, &subname2, -1);
+	break;
+      }
+      case TRACKLIST_ALBUM_COLUMN:
+      {
+        gtk_tree_model_get(model, a, TRACKLIST_TRACKNO_COLUMN, &subint1, -1);
+        gtk_tree_model_get(model, b, TRACKLIST_TRACKNO_COLUMN, &subint2, -1);
+        gtk_tree_model_get(model, a, TRACKLIST_TITLE_COLUMN, &subname1, -1);
+        gtk_tree_model_get(model, b, TRACKLIST_TITLE_COLUMN, &subname2, -1);
+      }
+    }
+    if (name1 != NULL || name2 != NULL)
+       ret = g_utf8_collate(name1,name2);
+    else if (!(name1 == NULL && name2 == NULL))
+       ret = (name1 == NULL ? -1 : 1);
+
+    if (ret == 0)
+       ret = subint1 - subint2;
+
+    if (ret == 0)
+    {
+      if (subname1 != NULL || subname2 != NULL)
+         ret = g_utf8_collate(subname1,subname2);
+      else if (!(name1 == NULL && name2 == NULL))
+         ret = (subname1 == NULL ? -1 : 1);
+    }
+    if (name1 != NULL) g_free(name1);
+    if (name2 != NULL) g_free(name2);
+    if (subname1 != NULL) g_free(subname1);
+    if (subname2 != NULL) g_free(subname2);
+    return ret;
+}
 
 static GtkListStore *create_hdliststore()
 {
   GtkListStore *hdliststore;
+  GtkTreeSortable *sortable;
 
   hdliststore = gtk_list_store_new (TRACKLIST_N_COLUMNS,
 				    G_TYPE_STRING,
@@ -465,12 +545,28 @@ static GtkListStore *create_hdliststore()
 				    G_TYPE_STRING, /*ID*/
 				    G_TYPE_STRING /* Folder */
 				    );
+
+  sortable = GTK_TREE_SORTABLE(hdliststore);
+
+  gtk_tree_sortable_set_sort_func(sortable, 
+	TRACKLIST_ARTIST_COLUMN, sort_iter_compare_func,
+        GINT_TO_POINTER(TRACKLIST_ARTIST_COLUMN), NULL);
+
+  gtk_tree_sortable_set_sort_func(sortable, 
+	TRACKLIST_TITLE_COLUMN, sort_iter_compare_func,
+        GINT_TO_POINTER(TRACKLIST_TITLE_COLUMN), NULL);
+
+  gtk_tree_sortable_set_sort_func(sortable, 
+	TRACKLIST_ALBUM_COLUMN, sort_iter_compare_func,
+        GINT_TO_POINTER(TRACKLIST_ALBUM_COLUMN), NULL);
+
   return hdliststore;
 }
 
 static GtkListStore *create_jbliststore()
 {
   GtkListStore *jbliststore;
+  GtkTreeSortable *sortable;
 
   jbliststore = gtk_list_store_new (TRACKLIST_N_COLUMNS,
 				    G_TYPE_STRING,
@@ -487,6 +583,21 @@ static GtkListStore *create_jbliststore()
 				    G_TYPE_STRING, /*ID*/
 				    G_TYPE_STRING /* Folder */
 				    );
+
+  sortable = GTK_TREE_SORTABLE(jbliststore);
+
+  gtk_tree_sortable_set_sort_func(sortable, 
+	TRACKLIST_ARTIST_COLUMN, sort_iter_compare_func,
+        GINT_TO_POINTER(TRACKLIST_ARTIST_COLUMN), NULL);
+
+  gtk_tree_sortable_set_sort_func(sortable, 
+	TRACKLIST_TITLE_COLUMN, sort_iter_compare_func,
+        GINT_TO_POINTER(TRACKLIST_TITLE_COLUMN), NULL);
+
+  gtk_tree_sortable_set_sort_func(sortable, 
+	TRACKLIST_ALBUM_COLUMN, sort_iter_compare_func,
+        GINT_TO_POINTER(TRACKLIST_ALBUM_COLUMN), NULL);
+
   return jbliststore;
 }
 
@@ -584,33 +695,46 @@ static gint hdlist_compare_func (GtkTreeModel *model,
     if (order == GTK_SORT_DESCENDING)
       retval = -retval;
   } else {
-    /* Dynamically decide how to handle different text types */
-    if (coltype == G_TYPE_STRING) {
-      gchar *text1;
-      gchar *text2;
+    switch(sort_column_id)
+    {
+      case TRACKLIST_ALBUM_COLUMN:
+      case TRACKLIST_ARTIST_COLUMN:
+      case TRACKLIST_TITLE_COLUMN:
+      {
+        retval = sort_iter_compare_func(model, a, b, GINT_TO_POINTER(sort_column_id));
+      }
+      break;
+      default:
+      {
+        /* Dynamically decide how to handle different text types */
+        if (coltype == G_TYPE_STRING) {
+           gchar *text1;
+           gchar *text2;
       
-      gtk_tree_model_get(model, a, sort_column_id, &text1, -1);
-      gtk_tree_model_get(model, b, sort_column_id, &text2, -1);
-      if (!text2)
-	retval = (text1 != NULL);
-      else if (!text1)
-	retval = -1;
-      else 
-	retval = g_strcasecmp(text1, text2);
-      g_free(text1);
-      g_free(text2);
-    } else if (coltype == G_TYPE_INT) {
-      guint int1;
-      guint int2;
+           gtk_tree_model_get(model, a, sort_column_id, &text1, -1);
+           gtk_tree_model_get(model, b, sort_column_id, &text2, -1);
+           if (!text2)
+	      retval = (text1 != NULL);
+           else if (!text1)
+	      retval = -1;
+           else 
+	      retval = g_strcasecmp(text1, text2);
+           g_free(text1);
+           g_free(text2);
+        } else if (coltype == G_TYPE_INT) {
+          guint int1;
+          guint int2;
 
-      gtk_tree_model_get(model, a, sort_column_id, &int1, -1);
-      gtk_tree_model_get(model, b, sort_column_id, &int2, -1);
-      if (int1 < int2)
-	retval = -1;
-      else if (int1 > int2)
-	retval = 1;
-      else
-	retval = 0;
+          gtk_tree_model_get(model, a, sort_column_id, &int1, -1);
+          gtk_tree_model_get(model, b, sort_column_id, &int2, -1);
+          if (int1 < int2)
+	    retval = -1;
+          else if (int1 > int2)
+	    retval = 1;
+          else
+	    retval = 0;
+	}
+      }
     }
   }
   return retval;
