@@ -85,14 +85,17 @@ typedef enum
 
 metadata_t *new_metadata_t();
 void destroy_metadata_t(metadata_t *meta);
-metadata_t *clone_metadata_t(metadata_t *meta);
-void dump_metadata_t(metadata_t *meta);
-gchar *compose_metadata_string(metadata_t *meta, gchar *template, gboolean filesystemsafe);
+metadata_t *clone_metadata_t(metadata_t const * const meta);
+void dump_metadata_t(metadata_t const * const meta);
+gchar *compose_metadata_string(metadata_t const * const meta, 
+			       gchar const * const template, 
+			       gboolean const filesystemsafe);
 metadata_t *get_metadata_from_model(GtkTreeModel *model, 
 				    GtkTreeIter *iter);
 metadata_t *get_data_metadata_from_model(GtkTreeModel *model, 
 					 GtkTreeIter *iter);
-void add_metadata_to_model(metadata_t *meta, listtype_t listtype);
+void add_metadata_to_model(metadata_t const * const meta, 
+			   listtype_t const listtype);
 GtkTreeSelection *get_metadata_selection(listtype_t listtype);
 GList *get_all_metadata_from_selection(listtype_t listtype);
 void destroy_metalist(GList *metalist);
