@@ -27,21 +27,56 @@ typedef struct {
 } eax_adjust_props_t;
 
 static gboolean playing = FALSE;
+static GSList *exclusive_patches = NULL;
+// Previous (since last callback) position in the file as percent
+static guint prevpos;
 
 /* option values */
 static void patch_changed (gpointer combo,
 			   gpointer data)
 {
+  GSList *tmplist;
+  gboolean was_exclusive = FALSE;
   guint effect = GPOINTER_TO_UINT(data);
 #if ! GTK_CHECK_VERSION(2,4,0)
   guint patch = gtk_option_menu_get_history(GTK_OPTION_MENU(combo));
 #else
   guint patch = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
 #endif
+  // See if it is exclusive
+  tmplist = exclusive_patches;
+  while (tmplist != NULL) {
+    if (combo == tmplist->data) {
+      was_exclusive = TRUE;
+    }
+    tmplist = g_slist_next(tmplist);
+  }
+  // OK it was exclusive, reset all other exclusive patches...
+  if (was_exclusive) {
+    tmplist = exclusive_patches;
+    // Reset all
+    while (tmplist != NULL) {
+      if (combo != tmplist->data) {
+	// Reset it
+#if ! GTK_CHECK_VERSION(2,4,0)
+	gtk_option_menu_set_history(GTK_OPTION_MENU(tmplist->data), 0);
+#else
+	gtk_combo_box_set_active(GTK_COMBO_BOX(tmplist->data), 0);
+#endif
+      }
+      tmplist = g_slist_next(tmplist);
+    }
+    // Then make sure it is set
+#if ! GTK_CHECK_VERSION(2,4,0)
+    gtk_option_menu_set_history(GTK_OPTION_MENU(combo), patch);
+#else
+    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), patch);
+#endif
+  }
   
   if (playing) {
     // Wire scalevalue to 0
-    g_print("Selecting patch %d for effect %04X\n", patch, effect);
+    // g_print("Selecting patch %d for effect %04X\n", patch, effect);
     jukebox_adjust_eax(effect, patch, 0);
   }
 }
@@ -163,8 +198,8 @@ static void adjust_scalevalue (GtkAdjustment *adj,
   gint16 scalevalue = (gint16) adj->value;
 
   if (playing) {
-    g_print("Setting scalevalue for effect %04X to %d\n", 
-	    adjprops->effect, scalevalue);
+    //g_print("Setting scalevalue for effect %04X to %d\n", 
+    //  adjprops->effect, scalevalue);
     jukebox_adjust_eax(adjprops->effect, 0, scalevalue);
   }
 }
@@ -174,7 +209,7 @@ static void close_scalevalue (gpointer data,
 {
   eax_adjust_props_t *adjprops = (eax_adjust_props_t *) data;
 
-  g_print("Destroying adjprops for %04X\n", adjprops->effect);
+  //g_print("Destroying adjprops for %04X\n", adjprops->effect);
   g_free(adjprops);
 }
 
@@ -254,6 +289,12 @@ static GtkWidget *get_eax_controls()
 #define VERTICAL   0x01
   u_int8_t last_direction = HORIZONTAL;
 
+  // Clear old exclusive list...
+  if (exclusive_patches != NULL) {
+    g_slist_free(exclusive_patches);
+    exclusive_patches = NULL;
+  }
+
   // Stuff all EAX stuff into this frame
   hbox = gtk_hbox_new(FALSE, 5);
 
@@ -271,7 +312,7 @@ static GtkWidget *get_eax_controls()
       // If this groups consists of scalevalues (we are guessing
       // that the first group member speaks for the rest), then create
       // a hbox, else (for patches etc) create a hbox.
-      if (eax->scaleable != 0x00) {
+      if (eax->type == NJB_EAX_SLIDER_CONTROL) {
 	group_box = gtk_hbox_new(FALSE, 0);
 	last_direction = HORIZONTAL;
       } else {
@@ -282,13 +323,13 @@ static GtkWidget *get_eax_controls()
     // Then each effect with it's label is stuffed in a vbox
     vbox = gtk_vbox_new(FALSE, 0);
     // onoff values
-    if (eax->selectable != 0x00 &&
-	eax->min_selectionvalue == 0x00 &&
-	eax->max_selectionvalue == 0x01) {
+    if (eax->type == NJB_EAX_FIXED_OPTION_CONTROL &&
+	eax->min_value == 0x00 &&
+	eax->max_value == 0x01) {
       guint effect = (guint) eax->number;
 
       checkbox = gtk_check_button_new_with_label(eax->name);
-      if (eax->current_selectionvalue != 0x00) {
+      if (eax->current_value != 0x00) {
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox), TRUE);
       }
       g_signal_connect(GTK_OBJECT(checkbox),
@@ -302,30 +343,35 @@ static GtkWidget *get_eax_controls()
       gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
       gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
       // Patch value combobox
-      if (eax->selectable != 0x00) {
+      if (eax->type == NJB_EAX_FIXED_OPTION_CONTROL) {
 	guint i;
 	GSList *names = NULL;
+	GtkWidget *patches;
 	
-	for(i = 0; i < eax->max_selectionvalue - eax->min_selectionvalue + 1; i++) {
-	  names = g_slist_append(names, g_strdup(eax->selection_names[i]));
+	for(i = 0; i < eax->max_value - eax->min_value + 1; i++) {
+	  names = g_slist_append(names, g_strdup(eax->option_names[i]));
 	}
-	GtkWidget *patches = create_patch_menu(eax->number,
-					       names,
-					       GTK_SIGNAL_FUNC(patch_changed),
-					       eax->current_selectionvalue);
+	patches = create_patch_menu(eax->number,
+				    names,
+				    GTK_SIGNAL_FUNC(patch_changed),
+				    eax->current_value);
 	gtk_box_pack_start(GTK_BOX(vbox), patches, TRUE, TRUE, 0);
+	if (eax->exclusive) {
+	  // Add to list of exclusive stuff
+	  exclusive_patches = g_slist_append(exclusive_patches, (gpointer) patches);
+	}
       }
       // Scale value slider
-      if (eax->scaleable != 0x00) {
+      if (eax->type == NJB_EAX_SLIDER_CONTROL) {
 	GtkWidget *slider;
 	
-	g_print("Scalevalue %s, min %d, max %d, current %d\n",
-		eax->name, eax->min_scalevalue, eax->max_scalevalue,
-		eax->current_scalevalue);
+	// g_print("Scalevalue %s, min %d, max %d, current %d\n",
+	//	eax->name, eax->min_scalevalue, eax->max_scalevalue,
+	//	eax->current_scalevalue);
 	slider = create_adjustment_control(eax->number,
-					   eax->min_scalevalue,
-					   eax->max_scalevalue,
-					   eax->current_scalevalue);
+					   eax->min_value,
+					   eax->max_value,
+					   eax->current_value);
 	gtk_box_pack_start(GTK_BOX(vbox), slider, TRUE, TRUE, 0);
 	gtk_widget_show(slider);
       }
@@ -343,6 +389,45 @@ static GtkWidget *get_eax_controls()
   return hbox;
 }
 
+static void adjust_position (GtkAdjustment *adj,
+			     gpointer data)
+{
+  // eax_adjust_props_t *adjprops = (eax_adjust_props_t *) data;
+  // gint16 scalevalue = adjprops->max+1 - ((gint16) adj->value);
+  gint16 pos = (gint16) adj->value;
+  // Calculate the byte position offset
+  guint delta = 0;
+  guint skipto = 0;
+  
+  // If we just passed a track boundary there is no delta,
+  // else we find out how much the position has changed.
+  if ((prevpos == 100 || prevpos == 99) && (pos == 0  || pos == 1)) {
+    delta = 0;
+  } else if (pos >= prevpos) {
+    delta = pos - prevpos;
+  } else if (pos < prevpos) {
+    delta = prevpos - pos;
+  }
+
+  // If someone has dragged the progress bar, delta will be more
+  // than 2 percent (let's say).
+  if (delta > 2) {
+    skipto = pos;
+  }
+
+  // If we're skipping backward or forward, execute this.
+  if (playing && skipto > 0) {
+    metadata_t *meta = jukebox_get_current_playing_metadata();
+    guint seconds = mmss_to_seconds(meta->length);
+    guint mspos = (seconds * 1000 * skipto) / 100;
+    g_print("Skipping to millisecond offset %d in track\n", mspos);
+    // Call NJB
+    jukebox_skip_songposition(mspos);
+  }
+  prevpos = pos;
+}
+
+
 /* Creates a complicated player window */
 void create_player_window(GList *metalist)
 {
@@ -368,7 +453,14 @@ void create_player_window(GList *metalist)
    * (upper - page_size). */
   adj = gtk_adjustment_new (0.0, 0.0, 101.0, 0.1, 1.0, 1.0);
   hscale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
-  gtk_widget_set_sensitive(hscale, FALSE);
+  // To make it possible to skip into the song
+  // gtk_widget_set_sensitive(hscale, FALSE);
+  g_signal_connect_data(GTK_OBJECT(adj),
+			"value_changed",
+			G_CALLBACK(adjust_position),
+			NULL, // data
+			NULL, // destructor
+			0);
   gtk_widget_set_usize (GTK_WIDGET (hscale), 200, 30);
   gtk_range_set_update_policy (GTK_RANGE (hscale),
 			       GTK_UPDATE_CONTINUOUS);
@@ -376,7 +468,11 @@ void create_player_window(GList *metalist)
   gtk_scale_set_value_pos (GTK_SCALE(hscale), GTK_POS_TOP);
   gtk_scale_set_draw_value (GTK_SCALE(hscale), FALSE);
   gtk_widget_show (hscale);
+#if GTK_CHECK_VERSION(2,6,0)
+  button =  gtk_button_new_from_stock(GTK_STOCK_MEDIA_STOP);
+#else
   button = gtk_button_new_with_label(_("STOP"));
+#endif
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), button,
 		      FALSE, FALSE, 0);
@@ -402,6 +498,9 @@ void create_player_window(GList *metalist)
   /* Manouver panel */
   hbox2 = gtk_hbox_new(FALSE, 0);
 
+#if GTK_CHECK_VERSION(2,6,0)
+  button =  gtk_button_new_from_stock(GTK_STOCK_MEDIA_PREVIOUS);
+#else
   button = gtk_button_new();
   hbox3 = gtk_hbox_new(FALSE, 0);
   arrow = gtk_arrow_new(GTK_ARROW_LEFT, GTK_SHADOW_NONE);
@@ -410,6 +509,7 @@ void create_player_window(GList *metalist)
   gtk_box_pack_start(GTK_BOX(hbox3), label, TRUE, TRUE, 0);
   gtk_widget_show(arrow);
   gtk_container_add(GTK_CONTAINER(button), hbox3);
+#endif
   g_signal_connect_object(GTK_OBJECT(button),
 			  "clicked",
 			  G_CALLBACK(prevbutton_click),
@@ -419,6 +519,9 @@ void create_player_window(GList *metalist)
   GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
   label3 = gtk_label_new("00:00:00");
   gtk_box_pack_start(GTK_BOX(hbox2), label3, TRUE, TRUE, 0);
+#if GTK_CHECK_VERSION(2,6,0)
+  button =  gtk_button_new_from_stock(GTK_STOCK_MEDIA_NEXT);
+#else
   button = gtk_button_new();
   hbox3 = gtk_hbox_new(FALSE, 0);
   label = gtk_label_new(_("NEXT"));
@@ -427,6 +530,7 @@ void create_player_window(GList *metalist)
   gtk_box_pack_start(GTK_BOX(hbox3), arrow, TRUE, TRUE, 0);
   gtk_widget_show(arrow);
   gtk_container_add(GTK_CONTAINER(button), hbox3);
+#endif
   g_signal_connect_object(GTK_OBJECT(button),
 			  "clicked",
 			  G_CALLBACK(nextbutton_click),
