@@ -27,7 +27,12 @@ Boston, MA 02111-1307, USA.
 #include "util.h"
 #include "player.h"
 #include "xfer.h"
+
+#if !GTK_CHECK_VERSION(2,6,0)
 #include <libgnomeui/libgnomeui.h>
+#else
+#define GNOME_DIALOG GTK_DIALOG
+#endif
 
 /*****************************************************************************
  * Here comes the jukebox transfer functions.
@@ -49,6 +54,7 @@ static void transfer_from_jukebox_to_hd (GtkButton *button,
   if (metalist != NULL) {
     GtkWidget *label1, *label2, *dialog, *button, *separator;
 
+#if !GTK_CHECK_VERSION(2,6,0)    
     dialog = gnome_dialog_new(_("Transfering tracks from jukebox library"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
@@ -56,6 +62,20 @@ static void transfer_from_jukebox_to_hd (GtkButton *button,
 				0,
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
+#else
+    dialog=gtk_dialog_new_with_buttons(_("Transfering tracks from jukebox library"),
+		    NULL,
+		    0,
+		    NULL);
+    button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+		    GTK_STOCK_CANCEL,
+		    0);
+    g_signal_connect(button,
+		    "clicked",
+		    G_CALLBACK(cancel_jukebox_operation_click),
+		    NULL);  
+#endif
+    
     g_signal_connect_object(GTK_OBJECT(dialog),
 			    "delete_event",
 			    G_CALLBACK(gtk_widget_destroy),
@@ -136,6 +156,7 @@ void transfer_from_hd_to_jukebox(GList *playlists)
 
 
     hd2jb_thread_args = (hd2jb_thread_arg_t *) malloc(sizeof (hd2jb_thread_arg_t));
+#if !GTK_CHECK_VERSION(2,6,0)
     dialog = gnome_dialog_new(_("Transfering tracks to jukebox library"),
 			      GNOME_STOCK_BUTTON_CANCEL,
 			      NULL);
@@ -143,6 +164,19 @@ void transfer_from_hd_to_jukebox(GList *playlists)
 				0,
 				G_CALLBACK(cancel_jukebox_operation_click),
 				NULL);
+#else
+     dialog=gtk_dialog_new_with_buttons(_("Transfering tracks to jukebox library"),
+		    NULL,
+		    0,
+		    NULL);
+    button = gtk_dialog_add_button(GTK_DIALOG(dialog),
+		    GTK_STOCK_CANCEL,
+		    0);
+    g_signal_connect(button,
+		    "clicked",
+		    G_CALLBACK(cancel_jukebox_operation_click),
+		    NULL);     
+#endif
     g_signal_connect_object(GTK_OBJECT(dialog),
 			    "delete_event",
 			    G_CALLBACK(gtk_widget_destroy),
